@@ -1,7 +1,7 @@
 /*****************************************************************************
  * Util.java
  *****************************************************************************
- * Copyright © 2011-2013 VLC authors and VideoLAN
+ * Copyright © 2011-2014 VLC authors and VideoLAN
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,34 +21,43 @@
 package org.videolan.vlc;
 
 import java.io.BufferedReader;
-import java.io.File;
+import java.io.BufferedWriter;
 import java.io.FileNotFoundException;
+import java.io.FileOutputStream;
 import java.io.FileReader;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
+import java.io.OutputStreamWriter;
 import java.text.DecimalFormat;
 import java.text.NumberFormat;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashSet;
+import java.util.List;
 import java.util.Locale;
 import java.util.StringTokenizer;
 
+import org.json.JSONArray;
+import org.json.JSONException;
 import org.videolan.libvlc.LibVLC;
 import org.videolan.libvlc.LibVlcException;
+import org.videolan.libvlc.LibVlcUtil;
+import org.videolan.libvlc.Media;
 
 import android.content.Context;
 import android.content.SharedPreferences;
+import android.content.SharedPreferences.Editor;
+import android.content.res.TypedArray;
+import android.database.sqlite.SQLiteFullException;
 import android.graphics.Bitmap;
-import android.net.Uri;
 import android.os.Environment;
 import android.preference.PreferenceManager;
 import android.telephony.TelephonyManager;
 import android.util.DisplayMetrics;
+import android.util.Log;
 import android.util.TypedValue;
 import android.view.Display;
-import android.view.View;
 import android.view.WindowManager;
 import android.widget.Toast;
 
@@ -63,7 +72,8 @@ public class Util {
         devicesWithoutNavBar.add("HTC One S");
         devicesWithoutNavBar.add("HTC One X");
         devicesWithoutNavBar.add("HTC One XL");
-        hasNavBar = isICSOrLater() && !devicesWithoutNavBar.contains(android.os.Build.MODEL);
+        hasNavBar = LibVlcUtil.isICSOrLater()
+                && !devicesWithoutNavBar.contains(android.os.Build.MODEL);
     }
 
     public static LibVLC getLibVlcInstance() throws LibVlcException {
@@ -80,17 +90,47 @@ public class Util {
         return instance;
     }
 
+    public static float[] getFloatArray(SharedPreferences pref, String key) {
+        float[] array = null;
+        String s = pref.getString(key, null);
+        if (s != null) {
+            try {
+                JSONArray json = new JSONArray(s);
+                array = new float[json.length()];
+                for (int i = 0; i < array.length; i++)
+                    array[i] = (float) json.getDouble(i);
+            } catch (JSONException e) {
+                e.printStackTrace();
+            }
+        }
+        return array;
+    }
+
+    public static void putFloatArray(Editor editor, String key, float[] array) {
+        try {
+            JSONArray json = new JSONArray();
+            for (float f : array)
+                json.put(f);
+            editor.putString("equalizer_values", json.toString());
+        } catch (JSONException e) {
+            e.printStackTrace();
+        }
+    }
+
     public static void updateLibVlcSettings(SharedPreferences pref) {
         LibVLC instance = LibVLC.getExistingInstance();
         if (instance == null)
             return;
 
-        instance.setIomx(pref.getBoolean("enable_iomx", false));
-        instance.setSubtitlesEncoding(pref.getString("subtitles_text_encoding", ""));
+        instance.setSubtitlesEncoding(pref.getString("subtitle_text_encoding", ""));
         instance.setTimeStretching(pref.getBoolean("enable_time_stretching_audio", false));
+        instance.setFrameSkip(pref.getBoolean("enable_frame_skip", false));
         instance.setChroma(pref.getString("chroma_format", ""));
         instance.setVerboseMode(pref.getBoolean("enable_verbose_mode", true));
 
+        if (pref.getBoolean("equalizer_enabled", false))
+            instance.setEqualizer(getFloatArray(pref, "equalizer_values"));
+
         int aout;
         try {
             aout = Integer.parseInt(pref.getString("aout", "-1"));
@@ -98,6 +138,13 @@ public class Util {
         catch (NumberFormatException nfe) {
             aout = -1;
         }
+        int vout;
+        try {
+        	vout = Integer.parseInt(pref.getString("vout", "-1"));
+        }
+        catch (NumberFormatException nfe) {
+        	vout = -1;
+        }
         int deblocking;
         try {
             deblocking = Integer.parseInt(pref.getString("deblocking", "-1"));
@@ -105,8 +152,23 @@ public class Util {
         catch(NumberFormatException nfe) {
             deblocking = -1;
         }
+        int hardwareAcceleration;
+        try {
+            hardwareAcceleration = Integer.parseInt(pref.getString("hardware_acceleration", "-1"));
+        }
+        catch(NumberFormatException nfe) {
+            hardwareAcceleration = -1;
+        }
+        int networkCaching = pref.getInt("network_caching_value", 0);
+        if(networkCaching > 60000)
+            networkCaching = 60000;
+        else if(networkCaching < 0)
+            networkCaching = 0;
         instance.setAout(aout);
+        instance.setVout(vout);
         instance.setDeblocking(deblocking);
+        instance.setNetworkCaching(networkCaching);
+        instance.setHardwareAcceleration(hardwareAcceleration);
     }
 
     /** Print an on-screen message to alert the user */
@@ -118,14 +180,6 @@ public class Util {
         toaster(context, stringId, Toast.LENGTH_SHORT);
     }
 
-    public static File URItoFile(String URI) {
-        return new File(Uri.decode(URI).replace("file://",""));
-    }
-
-    public static String URItoFileName(String URI) {
-        return URItoFile(URI).getName();
-    }
-
     public static String stripTrailingSlash(String s) {
         if( s.endsWith("/") && s.length() > 1 )
             return s.substring(0, s.length() - 1);
@@ -153,12 +207,56 @@ public class Util {
         }
     }
 
+    /**
+     * Writes the current app logcat to a file.
+     *
+     * @param filename The filename to save it as
+     * @throws IOException
+     */
+    public static void writeLogcat(String filename) throws IOException {
+        String[] args = { "logcat", "-v", "time", "-d" };
+
+        Process process = Runtime.getRuntime().exec(args);
+        InputStreamReader input = new InputStreamReader(
+                process.getInputStream());
+        OutputStreamWriter output = new OutputStreamWriter(
+                new FileOutputStream(filename));
+        BufferedReader br = new BufferedReader(input);
+        BufferedWriter bw = new BufferedWriter(output);
+        String line;
+
+        while ((line = br.readLine()) != null) {
+            bw.write(line);
+            bw.newLine();
+        }
+
+        bw.close();
+        output.close();
+        br.close();
+        input.close();
+    }
+
     /**
      * Convert time to a string
      * @param millis e.g.time/length from file
      * @return formated string (hh:)mm:ss
      */
-    public static String millisToString(long millis) {
+    public static String millisToString(long millis)
+    {
+        return millisToString(millis, false);
+    }
+
+    /**
+     * Convert time to a string
+     * @param millis e.g.time/length from file
+     * @return formated string "[hh]h[mm]min" / "[mm]min[s]s"
+     */
+    public static String millisToText(long millis)
+    {
+        return millisToString(millis, true);
+    }
+
+    private static String millisToString(long millis, boolean text) {
         boolean negative = millis < 0;
         millis = java.lang.Math.abs(millis);
 
@@ -172,10 +270,19 @@ public class Util {
         String time;
         DecimalFormat format = (DecimalFormat)NumberFormat.getInstance(Locale.US);
         format.applyPattern("00");
-        if (millis > 0) {
-            time = (negative ? "-" : "") + hours + ":" + format.format(min) + ":" + format.format(sec);
-        } else {
-            time = (negative ? "-" : "") + min + ":" + format.format(sec);
+        if (text) {
+            if (millis > 0)
+                time = (negative ? "-" : "") + hours + "h" + format.format(min) + "min";
+            else if (min > 0)
+                time = (negative ? "-" : "") + min + "min";
+            else
+                time = (negative ? "-" : "") + sec + "s";
+        }
+        else {
+            if (millis > 0)
+                time = (negative ? "-" : "") + hours + ":" + format.format(min) + ":" + format.format(sec);
+            else
+                time = (negative ? "-" : "") + min + ":" + format.format(sec);
         }
         return time;
     }
@@ -229,18 +336,48 @@ public class Util {
                 (width - (2 * left)), (height - (2 * top)));
     }
 
+    public static Bitmap getPictureFromCache(Media media)
+    {
+        // mPicture is not null only if passed through
+        // the ctor which is deprecated by now.
+        Bitmap b = media.getPicture();
+        if(b == null) {
+            BitmapCache cache = BitmapCache.getInstance();
+            Bitmap picture = cache.getBitmapFromMemCache(media.getLocation());
+            if(picture == null) {
+                /* Not in memcache:
+                 * serving the file from the database and
+                 * adding it to the memcache for later use.
+                 */
+                Context c = VLCApplication.getAppContext();
+                picture = MediaDatabase.getInstance(c).getPicture(c, media.getLocation());
+                cache.addBitmapToMemCache(media.getLocation(), picture);
+            }
+            return picture;
+        } else {
+            return b;
+        }
+    }
+
+    public static void setPicture(Context context, Media m, Bitmap p) {
+        Log.d(TAG, "Setting new picture for " + m.getTitle());
+        try {
+            MediaDatabase.getInstance(context).updateMedia(
+                m.getLocation(),
+                MediaDatabase.mediaColumn.MEDIA_PICTURE,
+                p);
+        } catch (SQLiteFullException e) {
+            Log.d(TAG, "SQLiteFullException while setting picture");
+        }
+        m.setPictureParsed(true);
+    }
+
     public static String getValue(String string, int defaultId)
     {
         return (string != null && string.length() > 0) ?
                 string : VLCApplication.getAppContext().getString(defaultId);
     }
 
-    public static void setItemBackground(View v, int position) {
-        v.setBackgroundResource(position % 2 == 0
-                ? R.drawable.background_item1
-                : R.drawable.background_item2);
-    }
-
     public static int convertPxToDp(int px) {
         WindowManager wm = (WindowManager)VLCApplication.getAppContext().
                 getSystemService(Context.WINDOW_SERVICE);
@@ -259,31 +396,6 @@ public class Util {
                          );
     }
 
-    public static boolean isFroyoOrLater()
-    {
-        return android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.FROYO;
-    }
-
-    public static boolean isGingerbreadOrLater()
-    {
-        return android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.GINGERBREAD;
-    }
-
-    public static boolean isHoneycombOrLater()
-    {
-        return android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.HONEYCOMB;
-    }
-
-    public static boolean isICSOrLater()
-    {
-        return android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.ICE_CREAM_SANDWICH;
-    }
-
-    public static boolean isJellyBeanOrLater()
-    {
-        return android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.JELLY_BEAN;
-    }
-
     public static boolean hasExternalStorage() {
         return Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED);
     }
@@ -309,37 +421,52 @@ public class Util {
         }
     }
 
-    public static String[] getStorageDirectories()
-    {
+    private static boolean StartsWith(String[] array, String text) {
+        for (String item : array)
+            if (text.startsWith(item))
+                return true;
+        return false;
+    }
+
+    public static String[] getStorageDirectories() {
         String[] dirs = null;
         BufferedReader bufReader = null;
+        ArrayList<String> list = new ArrayList<String>();
+        list.add(Environment.getExternalStorageDirectory().getPath());
+
+        List<String> typeWL = Arrays.asList("vfat", "exfat", "sdcardfs", "fuse");
+        List<String> typeBL = Arrays.asList("tmpfs");
+        String[] mountWL = { "/mnt", "/Removable" };
+        String[] mountBL = {
+                "/mnt/secure",
+                "/mnt/shell",
+                "/mnt/asec",
+                "/mnt/obb",
+                "/mnt/media_rw/extSdCard",
+                "/mnt/media_rw/sdcard",
+                "/storage/emulated" };
+        String[] deviceWL = {
+                "/dev/block/vold",
+                "/dev/fuse",
+                "/mnt/media_rw/extSdCard" };
+
         try {
             bufReader = new BufferedReader(new FileReader("/proc/mounts"));
-            ArrayList<String> list = new ArrayList<String>();
-            list.add(Environment.getExternalStorageDirectory().getPath());
             String line;
             while((line = bufReader.readLine()) != null) {
-                if(line.contains("vfat") || line.contains("exfat") ||
-                   line.contains("/mnt") || line.contains("/Removable")) {
-                    StringTokenizer tokens = new StringTokenizer(line, " ");
-                    String s = tokens.nextToken();
-                    s = tokens.nextToken(); // Take the second token, i.e. mount point
 
-                    if (list.contains(s))
-                        continue;
+                StringTokenizer tokens = new StringTokenizer(line, " ");
+                String device = tokens.nextToken();
+                String mountpoint = tokens.nextToken();
+                String type = tokens.nextToken();
 
-                    if (line.contains("/dev/block/vold")) {
-                        if (!line.startsWith("tmpfs") &&
-                            !line.startsWith("/dev/mapper") &&
-                            !s.startsWith("/mnt/secure") &&
-                            !s.startsWith("/mnt/shell") &&
-                            !s.startsWith("/mnt/asec") &&
-                            !s.startsWith("/mnt/obb")
-                            ) {
-                            list.add(s);
-                        }
-                    }
-                }
+                // skip if already in list or if type/mountpoint is blacklisted
+                if (list.contains(mountpoint) || typeBL.contains(type) || StartsWith(mountBL, mountpoint))
+                    continue;
+
+                // check that device is in whitelist, and either type or mountpoint is in a whitelist
+                if (StartsWith(deviceWL, device) && (typeWL.contains(type) || StartsWith(mountWL, mountpoint)))
+                    list.add(mountpoint);
             }
 
             dirs = new String[list.size()];
@@ -431,4 +558,17 @@ public class Util {
     public static boolean nullEquals(String s1, String s2) {
         return (s1 == null ? s2 == null : s1.equals(s2));
     }
+
+    /**
+     * Get a resource id from an attribute id.
+     * @param context
+     * @param attrId
+     * @return the resource id
+     */
+    public static int getResourceFromAttribute(Context context, int attrId) {
+        TypedArray a = context.getTheme().obtainStyledAttributes(new int[] {attrId});
+        int resId = a.getResourceId(0, 0);
+        a.recycle();
+        return resId;
+    }
 }
